##############################################################################
#
# Copyright (C) Zenoss, Inc. 2015, all rights reserved.
#
# This content is made available according to terms specified in
# License.zenoss under the directory where your Zenoss product is installed.
#
##############################################################################

from mock import Mock, sentinel

from Products.ZenTestCase.BaseTestCase import BaseTestCase

from ZenPacks.zenoss.Layer2.modeler.plugins \
    .zenoss.snmp.CDPLLDPDiscover import _extract_cdp_lldp_maps

cdpCacheEntry1 = {
    '10648.28': {
        'cdpCacheAddress': '\nW\xfe\x07',
        'cdpCacheAddressType': 1,
        'cdpCacheDeviceId': '08cc6843e573',
        'cdpCacheDevicePort': 'gi52',
        'cdpCacheNativeVLAN': 1,
        'cdpCachePlatform': 'asdf'
    },
}

lldpRemEntry1 = {
    '0.100.22': {
        'lldpRemPortDesc': '',
        'lldpRemPortId': 'gi52',
        'lldpRemSysDesc': '',
        'lldpRemSysName': 'asdf'
    },
}

cdpCacheEntry2 = {
    '68.4': {
        'cdpCachePlatform': 'N5K-C56128P',
        'cdpCacheAddressType': 1,
        'cdpCacheDevicePort': 'Ethernet1/34',
        'cdpCacheNativeVLAN': 1,
        'cdpCacheDeviceId': 'PVH00ADS02(FOC2048R0QV)',
        'cdpCacheAddress': '\n\xd4\x01\x05'
    },
    '61.3': {
        'cdpCachePlatform': 'N5K-C56128P',
        'cdpCacheAddressType': 1,
        'cdpCacheDevicePort': 'Ethernet1/34',
        'cdpCacheNativeVLAN': 1,
        'cdpCacheDeviceId': 'PVH00ADS01(FOC2048R0MF)',
        'cdpCacheAddress': '\n\xd4\x01\x04'
    },
}

lldpRemEntry2 = {
    '0.60.2': {
        'lldpRemSysName': 'PVH00ADS02',
        'lldpRemSysDesc': 'Cisco NX-OS(tm) n6000, Software (n6000-uk9), Version 7.3(7)N1(1), RELEASE SOFTWARE Copyright (c) 2002-2012, 2016-2017 by Cisco Systems, Inc. Compiled 1/26/2020 22:00:00',
        'lldpRemPortDesc': 'Ethernet1/34',
        'lldpRemPortId': 'Eth1/34'
    },
    '0.3.10': {
        'lldpRemSysName': 'AVX080739',
        'lldpRemSysDesc': '',
        'lldpRemPortDesc': '',
        'lldpRemPortId': '\xa0\t\xed\x08\x079'
    },
    '0.53.1': {
        'lldpRemSysName': 'PVH00ADS01',
        'lldpRemSysDesc': 'Cisco NX-OS(tm) n6000, Software (n6000-uk9), Version 7.3(7)N1(1), RELEASE SOFTWARE Copyright (c) 2002-2012, 2016-2017 by Cisco Systems, Inc. Compiled 1/26/2020 22:00:00',
        'lldpRemPortDesc': 'Ethernet1/34',
        'lldpRemPortId': 'Eth1/34'
    },
    '0.5.9': {
        'lldpRemSysName': '',
        'lldpRemSysDesc': '',
        'lldpRemPortDesc': '',
        'lldpRemPortId': '\xc44k{\xbe\xd3'
    },
    '0.6.11': {
        'lldpRemSysName': '',
        'lldpRemSysDesc': '',
        'lldpRemPortDesc': '',
        'lldpRemPortId': '\xdcJ>\x8b\xf4\xd5'
    },
    '0.1.4': {
        'lldpRemSysName': '',
        'lldpRemSysDesc': '',
        'lldpRemPortDesc': '',
        'lldpRemPortId': '\x88Q\xfb?a\x95'
    },
    '0.4.7': {
        'lldpRemSysName': 'AVX081116',
        'lldpRemSysDesc': '',
        'lldpRemPortDesc': '',
        'lldpRemPortId': '\xa0\t\xed\x08\x11\x16'
    },
    '0.2.8': {
        'lldpRemSysName': 'AVXB0C617',
        'lldpRemSysDesc': '',
        'lldpRemPortDesc': '',
        'lldpRemPortId': '\xd4xV\xb0\xc6\x17'
    },
}


class TestCDPLLDPDiscover(BaseTestCase):

    def test_extraction_of_both(self):
        res = _extract_cdp_lldp_maps({
            'cdpCacheEntry': cdpCacheEntry1,
            'lldpRemEntry': lldpRemEntry1
        })
        self.assertEqual(sorted(res), [{
            'description': '',
            'device_port': 'gi52',
            'id': 'lldp_0.100.22',
            'title': 'asdf'
        }, {
            'description': '',
            'device_port': 'gi52',
            'id': 'cdp_10648.28',
            'ip_address': '10.87.254.7',
            'location': '',
            'native_vlan': 1,
            'title': 'asdf'
        }])

    def test_cdp(self):
        res = _extract_cdp_lldp_maps({
            'cdpCacheEntry': cdpCacheEntry1,
        })
        self.assertEqual(res, [{
            'description': '',
            'device_port': 'gi52',
            'id': 'cdp_10648.28',
            'ip_address': '10.87.254.7',
            'location': '',
            'native_vlan': 1,
            'title': 'asdf'
        }])

    def test_lldp(self):
        res = _extract_cdp_lldp_maps({
            'lldpRemEntry': lldpRemEntry1
        })
        self.assertEqual(res, [{
            'description': '',
            'device_port': 'gi52',
            'id': 'lldp_0.100.22',
            'title': 'asdf'
        }])

    def test_hex_lldp_rem_port_id_encoding(self):
        res = _extract_cdp_lldp_maps({
            'lldpRemEntry': lldpRemEntry2,
            'cdpCacheEntry': cdpCacheEntry2,
        })

        self.assertEqual(res[5], {
                'description': '',
                'device_port': 'a009ed080739',
                'id': 'lldp_0.3.10',
                'title': 'AVX080739'
            }
        )


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCDPLLDPDiscover))
    return suite
